searchData={"items":[{"type":"module","title":"Slop.ExampleRouter.ExampleChatController","doc":"","ref":"Slop.ExampleRouter.ExampleChatController.html"},{"type":"function","title":"Slop.ExampleRouter.ExampleChatController.handle_chat/2","doc":"Process a chat request and return a response.\n\nThis callback should be implemented to handle chat requests. It should return\neither a map that will be encoded to JSON, or a connection to support streaming.","ref":"Slop.ExampleRouter.ExampleChatController.html#handle_chat/2"},{"type":"module","title":"Slop.ExampleRouter.ExampleInfoController","doc":"","ref":"Slop.ExampleRouter.ExampleInfoController.html"},{"type":"function","title":"Slop.ExampleRouter.ExampleInfoController.get_info/1","doc":"Get information about the SLOP server.\n\nThis callback should return a map containing information about the server.","ref":"Slop.ExampleRouter.ExampleInfoController.html#get_info/1"},{"type":"module","title":"Slop","doc":"SLOP - Simple Language Open Protocol for Elixir / Phoenix\n\nSLOP defines a simple, standardized set of HTTP endpoints for AI service interactions.\nIt provides a modular, easy-to-implement REST API pattern for building AI applications.","ref":"Slop.html"},{"type":"module","title":"Key Endpoints - Slop","doc":"* `/chat` - For sending messages to and receiving responses from AI models\n* `/tools` - For listing available tools and executing them\n* `/memory` - For storing and retrieving conversation history or other data\n* `/resources` - For accessing knowledge, files, and other resources\n* `/pay` - For handling payment-related operations\n* `/info` - For exposing server metadata and capabilities","ref":"Slop.html#module-key-endpoints"},{"type":"module","title":"Usage in Phoenix Router - Slop","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n\n  # Your existing pipeline definitions\n  pipeline :api do\n    plug :accepts, [\"json\"]\n  end\n\n  # Use the SLOP router in a scope\n  scope \"/api/slop\", MyAppWeb do\n    pipe_through :api\n    use Slop.Router, controllers: [\n      chat: MyAppWeb.SlopChatController,\n      tools: MyAppWeb.SlopToolsController,\n      memory: MyAppWeb.SlopMemoryController,\n      resources: MyAppWeb.SlopResourcesController,\n      pay: MyAppWeb.SlopPayController,\n      info: MyAppWeb.SlopInfoController\n    ]\n  end\nend\n```","ref":"Slop.html#module-usage-in-phoenix-router"},{"type":"module","title":"Implementing Controllers - Slop","doc":"```elixir\ndefmodule MyAppWeb.SlopChatController do\n  use Slop.Controller, :chat\n\n  # Override the default implementation\n  def handle_chat(_conn, params) do\n    messages = params[\"messages\"] || []\n\n    # Your AI integration code here\n    # ...\n\n    %{\n      messages: messages,\n      response: \"Hello from SLOP!\"\n    }\n  end\nend\n```\n\nFor more information on the SLOP protocol, visit https://github.com/agnt-gg/slop","ref":"Slop.html#module-implementing-controllers"},{"type":"macro","title":"Slop.__using__/1","doc":"Use in Phoenix Router to add SLOP endpoints to your application.\n\nThis is a convenience function that delegates to Slop.Router.","ref":"Slop.html#__using__/1"},{"type":"macro","title":"Example - Slop.__using__/1","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n\n  scope \"/api/slop\", MyAppWeb do\n    pipe_through :api\n    use Slop, controllers: [\n      chat: MyAppWeb.SlopChatController,\n      info: MyAppWeb.SlopInfoController\n    ]\n  end\nend\n```\n\nSee `Slop.Router` for more details on the available options.","ref":"Slop.html#__using__/1-example"},{"type":"module","title":"Slop.Controller","doc":"Provides default controller implementations for SLOP (Simple Language Open Protocol) endpoints.\n\nThis module can be used to quickly implement SLOP-compliant controllers in a Phoenix application\nby providing default implementations for all the standard SLOP endpoints.","ref":"Slop.Controller.html"},{"type":"module","title":"Usage - Slop.Controller","doc":"```elixir\ndefmodule MyAppWeb.SlopChatController do\n  use Slop.Controller, :chat\n\n  # Override the default implementation\n  def handle_chat(conn, params) do\n    # Your custom chat implementation\n    # ...\n\n    json(conn, %{response: \"Hello from SLOP!\"})\n  end\nend\n```\n\nYou can also implement only specific callbacks:\n\n```elixir\ndefmodule MyAppWeb.SlopInfoController do\n  use Slop.Controller, :info\n\n  # The default implementation will be used for other callbacks\n  def get_info(_conn) do\n    %{\n      name: \"My Custom SLOP Server\",\n      version: \"1.0.0\",\n      endpoints: [\"chat\", \"info\"],\n      models: [\"gpt-4\", \"claude-3\"]\n    }\n  end\nend","ref":"Slop.Controller.html#module-usage"},{"type":"macro","title":"Slop.Controller.__using__/1","doc":"Defines the controller implementation for a specific SLOP endpoint type.","ref":"Slop.Controller.html#__using__/1"},{"type":"macro","title":"Options - Slop.Controller.__using__/1","doc":"* `:chat` - Implements a controller for the /chat endpoint\n* `:tools` - Implements a controller for the /tools endpoints\n* `:memory` - Implements a controller for the /memory endpoints\n* `:resources` - Implements a controller for the /resources endpoints\n* `:pay` - Implements a controller for the /pay endpoint\n* `:info` - Implements a controller for the /info endpoint","ref":"Slop.Controller.html#__using__/1-options"},{"type":"module","title":"Slop.Router","doc":"A Phoenix router extension for implementing SLOP (Simple Language Open Protocol) endpoints.\n\nThis module provides a macro that can be used in a Phoenix router to quickly set up\nthe standard SLOP endpoints for AI interactions.","ref":"Slop.Router.html"},{"type":"module","title":"Usage - Slop.Router","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n\n  # Your existing pipeline definitions\n  pipeline :api do\n    plug :accepts, [\"json\"]\n  end\n\n  # Use the SLOP router in a scope\n  scope \"/api/slop\", MyAppWeb do\n    pipe_through :api\n    use Slop.Router, controllers: [\n      chat: MyAppWeb.SlopChatController,\n      tools: MyAppWeb.SlopToolsController,\n      memory: MyAppWeb.SlopMemoryController,\n      resources: MyAppWeb.SlopResourcesController,\n      pay: MyAppWeb.SlopPayController,\n      info: MyAppWeb.SlopInfoController\n    ]\n  end\nend\n```","ref":"Slop.Router.html#module-usage"},{"type":"module","title":"Configuration - Slop.Router","doc":"The `use Slop.Router` macro accepts a keyword list with the following options:\n\n* `:controllers` - A map of SLOP endpoint types to controller modules that will handle\n  the corresponding requests. You only need to specify the controllers for the endpoints\n  you want to implement.","ref":"Slop.Router.html#module-configuration"},{"type":"module","title":"Slop.Client","doc":"A client for consuming SLOP (Simple Language Open Protocol) APIs.\n\nThis module provides functions for interacting with SLOP servers, making it\neasy to send requests to the standard SLOP endpoints.","ref":"Slop.Client.html"},{"type":"module","title":"Usage - Slop.Client","doc":"```elixir\n# Initialize a client\nclient = Slop.Client.new(\"https://my-slop-server.com\")\n\n# Get server information\n{:ok, info} = Slop.Client.info(client)\n\n# Send a chat message\n{:ok, response} = Slop.Client.chat(client, [\n  %{role: \"user\", content: \"Hello, how are you?\"}\n])\n\n# List available tools\n{:ok, tools} = Slop.Client.list_tools(client)\n\n# Execute a tool\n{:ok, result} = Slop.Client.execute_tool(client, \"calculator\", %{\n  expression: \"2 + 2\"\n})\n```","ref":"Slop.Client.html#module-usage"},{"type":"function","title":"Slop.Client.chat/2","doc":"Sends a chat message to the SLOP server.","ref":"Slop.Client.html#chat/2"},{"type":"function","title":"Examples - Slop.Client.chat/2","doc":"```elixir\n{:ok, response} = Slop.Client.chat(client, [\n  %{role: \"user\", content: \"Hello, how are you?\"}\n])\n\n# response = %{\n#   \"response\" => \"I'm doing well, thank you for asking!\",\n#   \"messages\" => [\n#     %{\"role\" => \"user\", \"content\" => \"Hello, how are you?\"},\n#     %{\"role\" => \"assistant\", \"content\" => \"I'm doing well, thank you for asking!\"}\n#   ]\n# }\n```","ref":"Slop.Client.html#chat/2-examples"},{"type":"function","title":"Slop.Client.execute_tool/3","doc":"Executes a tool on the SLOP server.","ref":"Slop.Client.html#execute_tool/3"},{"type":"function","title":"Examples - Slop.Client.execute_tool/3","doc":"```elixir\n{:ok, result} = Slop.Client.execute_tool(client, \"calculator\", %{\n  expression: \"2 + 2\"\n})\n\n# result = %{\n#   \"id\" => \"calculator\",\n#   \"result\" => \"The result is 4\"\n# }\n```","ref":"Slop.Client.html#execute_tool/3-examples"},{"type":"function","title":"Slop.Client.get_resource/2","doc":"Gets a specific resource from the SLOP server.","ref":"Slop.Client.html#get_resource/2"},{"type":"function","title":"Examples - Slop.Client.get_resource/2","doc":"```elixir\n{:ok, %{\"resource\" => resource}} = Slop.Client.get_resource(client, \"article-1\")\n\n# resource = %{\n#   \"id\" => \"article-1\",\n#   \"title\" => \"Article 1\",\n#   \"content\" => \"...\",\n#   ...\n# }\n```","ref":"Slop.Client.html#get_resource/2-examples"},{"type":"function","title":"Slop.Client.info/1","doc":"Gets information about the SLOP server.","ref":"Slop.Client.html#info/1"},{"type":"function","title":"Examples - Slop.Client.info/1","doc":"```elixir\n{:ok, info} = Slop.Client.info(client)\n\n# info = %{\n#   \"name\" => \"Example SLOP Server\",\n#   \"version\" => \"1.0.0\",\n#   \"endpoints\" => [\"chat\", \"info\"],\n#   ...\n# }\n```","ref":"Slop.Client.html#info/1-examples"},{"type":"function","title":"Slop.Client.list_resources/1","doc":"Gets resources from the SLOP server.","ref":"Slop.Client.html#list_resources/1"},{"type":"function","title":"Examples - Slop.Client.list_resources/1","doc":"```elixir\n{:ok, %{\"resources\" => resources}} = Slop.Client.list_resources(client)\n\n# resources = [\n#   %{\n#     \"id\" => \"article-1\",\n#     \"title\" => \"Article 1\",\n#     ...\n#   },\n#   ...\n# ]\n```","ref":"Slop.Client.html#list_resources/1-examples"},{"type":"function","title":"Slop.Client.list_tools/1","doc":"Lists the tools available on the SLOP server.","ref":"Slop.Client.html#list_tools/1"},{"type":"function","title":"Examples - Slop.Client.list_tools/1","doc":"```elixir\n{:ok, %{\"tools\" => tools}} = Slop.Client.list_tools(client)\n\n# tools = [\n#   %{\n#     \"id\" => \"calculator\",\n#     \"name\" => \"Calculator\",\n#     \"description\" => \"Perform mathematical calculations\",\n#     ...\n#   },\n#   ...\n# ]\n```","ref":"Slop.Client.html#list_tools/1-examples"},{"type":"function","title":"Slop.Client.new/2","doc":"Creates a new SLOP client.","ref":"Slop.Client.html#new/2"},{"type":"function","title":"Options - Slop.Client.new/2","doc":"* `:headers` - Additional headers to include in requests\n* `:http_client` - The HTTP client module to use (default: `HTTPoison`)","ref":"Slop.Client.html#new/2-options"},{"type":"function","title":"Examples - Slop.Client.new/2","doc":"```elixir\n# Basic client\nclient = Slop.Client.new(\"https://my-slop-server.com\")\n\n# Client with authentication\nclient = Slop.Client.new(\"https://my-slop-server.com\",\n  headers: [{\"Authorization\", \"Bearer my-token\"}]\n)\n```","ref":"Slop.Client.html#new/2-examples"},{"type":"function","title":"Slop.Client.search_resources/2","doc":"Searches for resources on the SLOP server.","ref":"Slop.Client.html#search_resources/2"},{"type":"function","title":"Examples - Slop.Client.search_resources/2","doc":"```elixir\n{:ok, %{\"results\" => results}} = Slop.Client.search_resources(client, \"example\")\n\n# results = [\n#   %{\n#     \"id\" => \"article-1\",\n#     \"title\" => \"Example Article\",\n#     \"score\" => 0.9,\n#     ...\n#   },\n#   ...\n# ]\n```","ref":"Slop.Client.html#search_resources/2-examples"},{"type":"type","title":"Slop.Client.t/0","doc":"","ref":"Slop.Client.html#t:t/0"},{"type":"module","title":"Slop.Streaming","doc":"Provides helpers for streaming responses in SLOP protocol endpoints.\n\nThe SLOP protocol supports streaming responses through various mechanisms:\n\n1. Server-Sent Events (SSE)\n2. Chunked transfers\n3. WebSockets\n\nThis module provides helpers for implementing streaming in Phoenix controllers\nthat implement SLOP endpoints.","ref":"Slop.Streaming.html"},{"type":"module","title":"Usage - Slop.Streaming","doc":"```elixir\ndefmodule MyAppWeb.SlopChatController do\n  use Slop.Controller, :chat\n  import Slop.Streaming\n\n  def handle_chat(conn, params) do\n    # Start streaming response\n    conn = start_streaming(conn)\n\n    # Process messages and stream chunks\n    messages = params[\"messages\"] || []\n\n    # Stream back tokens one by one (simulating AI response)\n    response = \"Hello, I am an AI assistant. How can I help you today?\"\n\n    for word <- String.split(response) do\n      :timer.sleep(100) # Simulate thinking time\n      stream_chunk(conn, %{\n        type: \"content\",\n        content: word <> \" \"\n      })\n    end\n\n    # Signal completion\n    stream_chunk(conn, %{\n      type: \"done\",\n      content: \"\"\n    })\n\n    # Return the connection\n    conn\n  end\nend\n```","ref":"Slop.Streaming.html#module-usage"},{"type":"function","title":"Slop.Streaming.start_chunked_streaming/1","doc":"Starts a chunked transfer response for streaming in simple HTTP chunked mode.\n\nReturns a connection that's ready for streaming chunks.","ref":"Slop.Streaming.html#start_chunked_streaming/1"},{"type":"function","title":"Slop.Streaming.start_streaming/1","doc":"Starts streaming a response using Server-Sent Events (SSE).\n\nReturns a connection that's ready for streaming chunks.","ref":"Slop.Streaming.html#start_streaming/1"},{"type":"function","title":"Slop.Streaming.stream_chunk/2","doc":"Sends a chunk of data in the standard SLOP streaming format.\n\nThe chunk should be a map that will be converted to JSON. The map should\ninclude a `type` field that indicates the type of chunk:\n\n- \"content\" - A piece of content being streamed\n- \"done\" - Signals that the stream is complete\n- \"error\" - Indicates an error occurred\n- \"thinking\" - Indicates the AI is processing (optional)\n- \"tool_start\" - Indicates a tool is being called (optional)\n- \"tool_result\" - Contains the result of a tool call (optional)","ref":"Slop.Streaming.html#stream_chunk/2"},{"type":"function","title":"Examples - Slop.Streaming.stream_chunk/2","doc":"```elixir\nstream_chunk(conn, %{type: \"content\", content: \"Hello, \"})\nstream_chunk(conn, %{type: \"content\", content: \"world!\"})\nstream_chunk(conn, %{type: \"done\", content: \"\"})\n```","ref":"Slop.Streaming.html#stream_chunk/2-examples"},{"type":"function","title":"Slop.Streaming.stream_response/3","doc":"Helper for streaming a complete AI response word by word.\n\nThis function takes care of the streaming logic, including starting the stream,\nsending content chunks, and ending the stream.","ref":"Slop.Streaming.html#stream_response/3"},{"type":"function","title":"Example - Slop.Streaming.stream_response/3","doc":"```elixir\ndef handle_chat(conn, params) do\n  messages = params[\"messages\"] || []\n\n  # In a real implementation, call an AI API that streams tokens\n  response = \"Hello, I am an AI assistant. How can I help you today?\"\n\n  stream_response(conn, response, fn word, _index ->\n    :timer.sleep(100) # Simulate thinking time\n    word <> \" \"\n  end)\nend","ref":"Slop.Streaming.html#stream_response/3-example"},{"type":"module","title":"Slop.ExampleRouter","doc":"An example of how to use the Slop module in a Phoenix router.\n\nThis is not meant to be used directly, but to serve as a reference for\nhow to implement SLOP in your own Phoenix application.","ref":"Slop.ExampleRouter.html"},{"type":"function","title":"Slop.ExampleRouter.example_router_config/0","doc":"","ref":"Slop.ExampleRouter.html#example_router_config/0"},{"type":"extras","title":"SLOP for Phoenix","doc":"# SLOP for Phoenix\n\nA Phoenix router extension that implements the [SLOP (Simple Language Open Protocol)](https://github.com/agnt-gg/slop) for AI service interactions.","ref":"readme.html"},{"type":"extras","title":"Installation - SLOP for Phoenix","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `slop` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:slop, \"~> 0.1.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Usage - SLOP for Phoenix","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"In Your Phoenix Router - SLOP for Phoenix","doc":"Add SLOP endpoints to your Phoenix router:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n  \n  pipeline :api do\n    plug :accepts, [\"json\"]\n  end\n  \n  scope \"/api/slop\", MyAppWeb do\n    pipe_through :api\n    use Slop, controllers: [\n      chat: MyAppWeb.SlopChatController,\n      tools: MyAppWeb.SlopToolsController,\n      info: MyAppWeb.SlopInfoController\n    ]\n  end\nend\n```","ref":"readme.html#in-your-phoenix-router"},{"type":"extras","title":"Creating Controllers - SLOP for Phoenix","doc":"Create controllers that implement the SLOP endpoints:\n\n```elixir\ndefmodule MyAppWeb.SlopChatController do\n  use Slop.Controller, :chat\n  \n  # Override the default implementation\n  def handle_chat(_conn, params) do\n    messages = params[\"messages\"] || []\n    \n    # Your AI integration code here\n    response = \"Hello from SLOP!\"\n    \n    %{\n      messages: messages,\n      response: response\n    }\n  end\nend\n\ndefmodule MyAppWeb.SlopInfoController do\n  use Slop.Controller, :info\n  \n  def get_info(_conn) do\n    %{\n      name: \"My SLOP Server\",\n      version: \"1.0.0\",\n      endpoints: [\"chat\", \"tools\", \"info\"],\n      description: \"A Phoenix implementation of the SLOP protocol\",\n      models: [\"gpt-3.5\", \"gpt-4\", \"claude-3\"]\n    }\n  end\nend\n```","ref":"readme.html#creating-controllers"},{"type":"extras","title":"Tools Controller Example - SLOP for Phoenix","doc":"Create a tools controller to expose AI tools:\n\n```elixir\ndefmodule MyAppWeb.SlopToolsController do\n  use Slop.Controller, :tools\n  \n  def list_tools(_conn) do\n    [\n      %{\n        id: \"weather\",\n        name: \"Weather Lookup\",\n        description: \"Look up the current weather for a location\",\n        parameters: %{\n          type: \"object\",\n          required: [\"location\"],\n          properties: %{\n            location: %{\n              type: \"string\",\n              description: \"The city and state or country\"\n            }\n          }\n        }\n      },\n      %{\n        id: \"calculator\",\n        name: \"Calculator\",\n        description: \"Perform mathematical calculations\",\n        parameters: %{\n          type: \"object\",\n          required: [\"expression\"],\n          properties: %{\n            expression: %{\n              type: \"string\",\n              description: \"The mathematical expression to evaluate\"\n            }\n          }\n        }\n      }\n    ]\n  end\n  \n  def execute_tool(_conn, params) do\n    # Identify which tool to run based on parameters\n    case params do\n      %{\"tool_id\" => \"weather\", \"params\" => %{\"location\" => location}} ->\n        # In a real implementation, you would call a weather API\n        %{\n          id: \"weather\",\n          result: \"The weather in #{location} is sunny and 72°F\"\n        }\n        \n      %{\"tool_id\" => \"calculator\", \"params\" => %{\"expression\" => expression}} ->\n        # In a real implementation, you would safely evaluate the expression\n        %{\n          id: \"calculator\",\n          result: \"The result is 42\" # Just a placeholder\n        }\n        \n      _ ->\n        %{error: \"Invalid tool or parameters\"}\n    end\n  end\n  \n  def execute_specific_tool(_conn, tool_id, params) do\n    # Handle a specific tool by ID\n    case tool_id do\n      \"weather\" ->\n        location = params[\"params\"][\"location\"]\n        %{\n          id: \"weather\",\n          result: \"The weather in #{location} is sunny and 72°F\"\n        }\n        \n      \"calculator\" ->\n        expression = params[\"params\"][\"expression\"]\n        %{\n          id: \"calculator\",\n          result: \"The result is 42\" # Just a placeholder\n        }\n        \n      _ ->\n        %{error: \"Unknown tool: #{tool_id}\"}\n    end\n  end\nend\n```","ref":"readme.html#tools-controller-example"},{"type":"extras","title":"Endpoint Modules - SLOP for Phoenix","doc":"All SLOP endpoints are optional. You only need to implement the ones your application requires.\n\n| Endpoint   | Controller Type | Description                           |\n|------------|----------------|---------------------------------------|\n| `/chat`    | `:chat`        | Send messages to and receive responses from AI models |\n| `/tools`   | `:tools`       | List available tools and execute them |\n| `/memory`  | `:memory`      | Store and retrieve conversation history |\n| `/resources` | `:resources` | Access knowledge, files, and other resources |\n| `/pay`     | `:pay`         | Handle payment operations |\n| `/info`    | `:info`        | Expose server metadata and capabilities |","ref":"readme.html#endpoint-modules"},{"type":"extras","title":"SLOP Protocol - SLOP for Phoenix","doc":"For more details on the SLOP protocol, visit [https://github.com/agnt-gg/slop](https://github.com/agnt-gg/slop).","ref":"readme.html#slop-protocol"},{"type":"extras","title":"License - SLOP for Phoenix","doc":"MIT License","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}